#################################################################################
# Nombre del programa:	ESOC2023_IMPORTAR_DATA.R                                          
# Autor:              	IECA
# Versión:            	1.0
# Última modificaciÓn: 	09 Agosto 2023
#                                                                                
# Descripción: 
#	Este programa procesa un fichero de microdatos (md_ESOC2023.dat)
#
# Entrada:                                                           
#     - Archivo de microdatos: 	md_ESOC2023.dat
# Salida: 
#     - Archivo de microdatos en formato data.frame de R: md_ESOC2023
#                                                                                
#################################################################################

####################    Asignación de parámetros    #######################
#Recogemos la ruta del script en la que se encuentra el fichero de microdatos

setwd("C:/Microdatos_ESOC2023_ASCII")
fichero_micro <- "md_ESOC2023.dat"

#################################################################################

assign("flag_num", 0, envir = .GlobalEnv)

atencion = function(mensaje){
  cat(mensaje)
  assign("flag_num", 1, envir = .GlobalEnv)
  
}

####################     INICIO     #########################
start.time <- Sys.time()
cat("\n")
cat("\n Inicio: ")
print.Date(start.time)
t0 <- proc.time()

nombresVarbls <- c("idp","fep","sexo","edad","nacionalidad","cpro","grurba","compra1","compra2","compra3","compra4",
                   "compra5","donde1","donde2","donde3","donde4","donde5","como","alides1","alides2","alides3",
                   "alides4","alides5","alides6","respalim","acaract1","acaract2","acaract3","acaract4","acaract5",
                   "acaract6","acaract7","fresco","higiene1","higiene2","higiene3","higiene4","higiene5","higiene6",
                   "higiene7","pimport1","pimport2","pimport3","pimport4","pimport5","pimport6","resphig","rnueva",
                   "rcaract1","rcaract2","rcaract3","rcaract4","rcaract5","rcaract6","rcaract7","rdes","rnousa",
                   "resprop","medida1","medida2","medida3","medida4","medida5","medida6","medida7","medida8","noreduce1",
                   "noreduce2","noreduce3","noreduce4","noreduce5","co2","realiza","vdispo1","vdispo2","vdispo3","vdispo4",
                   "vdispo5","aguso1","aguso2","aguso3","aguso4","mtrans1","mtrans2","mtrans3","mtrans4","mtrans5","mtrans6",
                   "mtrans7","mtrans8","mmotiv1","mmotiv2","mmotiv3","mmotiv4","mmotiv5","mmotiv6","mmotiv7","mmotiv8",
                   "actitud","mambient","afecta","actdigi","nososte1","nososte2","nososte3","nososte4","nososte5","nososte6",
                   "nososte7","confia","propos1","propos2","propos3","propos4","propos5","propos6","propos7","propos8","nperson",
                   "marco","tipoh","cest","sactual","ttrabajo","cno","ing")



posiciones    <-c(4,15,1,2,1,2,1,1,1,1,1,1,
                  1,1,1,1,1,1,1,1,1,1,1,1,
                  1,1,1,1,1,1,1,1,1,1,1,1,
                  1,1,1,1,1,1,1,1,1,1,1,1,
                  2,2,2,2,2,2,2,1,1,2,1,1,
                  1,1,1,1,1,1,2,2,2,2,2,1,
                  1,2,2,2,2,2,1,1,1,1,1,1,
                  1,1,1,1,1,1,2,2,2,2,2,2,
                  2,2,1,1,2,2,1,1,1,1,1,1,
                  1,1,1,1,1,1,1,1,1,1,1,1,
                  1,1,2,2,2,2)


tipo          <- c("A","N","A","N","A","A","A","A","A","A","A","A",
                   "A","A","A","A","A","A","A","A","A","A","A","A",
                   "A","A","A","A","A","A","A","A","A","A","A","A",
                   "A","A","A","A","A","A","A","A","A","A","A","A",
                   "A","A","A","A","A","A","A","A","A","A","A","A",
                   "A","A","A","A","A","A","A","A","A","A","A","A",
                   "A","A","A","A","A","A","A","A","A","A","A","A",
                   "A","A","A","A","A","A","A","A","A","A","A","A",
                   "A","A","A","A","N","A","A","A","A","A","A","A",
                   "A","A","A","A","A","A","A","A","A","A","N","N",
                   "A","A","A","A","A","A")
                   
  

  

tamanio       <- length(nombresVarbls)


# Lectura del fichero de microdatos (MICROD)

  #Capturamos las columnas con su tipo de dato
  tipDatos <- as.vector(sapply(tipo, function(x){
    if(identical(x, "A"))
      "character"
    else{
      if(identical(x, "N"))
        "numeric"
    }
  }
  )
  )
  # Lectura del fichero de microdatos (MICROD), decimales con coma  
  tryCatch((df1 <- read.fwf(file = fichero_micro, widths= posiciones, colClasses=tipDatos)), error=function(e)
    stop(paste("Error. No se encuentra el fichero: ", e, fichero_micro,". Saliendo de la ejecuciÃ³n...", sep = "")))
  
#Aplicamos los nombres de la cabecera del registro

names(df1) <- nombresVarbls
df1$fep=df1$fep/100000000000
md_ESOC2023 <- df1






#Liberacion de memoria y aclaraciÃ³n de variables 
#Values
rm(flag_num,nombresVarbls,posiciones,tamanio,df1)
rm(tipDatos)


# Mensaje final ##########################################
end.time <- Sys.time()
cat("\n")
cat("\n Fin del proceso de lectura: ")
print.Date(end.time)

TTotal <- proc.time() - t0
tiempo <- TTotal[3]

if(tiempo < 60){
  cat(paste("\n Tiempo transcurrido:", format(round(tiempo, 2), nsmall = 2), "segundos"))
}else{
  if(tiempo< 3600 & tiempo >= 60){
    cat(paste("\n Tiempo transcurrido:", format(round(tiempo/60, 2), nsmall = 2), "minutos"))
  }else{
    cat(paste("\n Tiempo transcurrido:", format(round(tiempo/3600, 2), nsmall = 2), "horas"))
  }
}
